SystemOrganization addCategory: #'Seaside-Swazoo'!

Object subclass: #SwazooSeaside
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Swazoo'!

!SwazooSeaside commentStamp: 'pmm 11/17/2007 14:50' prior: 0!
I offer the same functionality as WAKom for Swazoo 2.

You can start and stop me with:
SwazooSeaside startOn: 8080
SwazooSeaside stop
which listens to http://127.0.0.1:8080/seaside/

Encoding can be switched on with:
SwazooSeaside doEncoding: true

You can use a specific encoding with:
SwazooSeaside encoding: 'utf-8'!

!SwazooSeaside class methodsFor: 'service' stamp: 'pmm 11/17/2007 14:43'!
doEncoding: aBoolean
	"toggle encoding mode
	with encoding on you will end up with mulit byte strings
	with encoding off you will end up with strings where each charachter is only a byte"
	SeasideSite doEncoding: aBoolean! !

!SwazooSeaside class methodsFor: 'service' stamp: 'pmm 11/17/2007 14:44'!
encoding: aStringOrNil
	"sets the encoding of the server"
	SeasideSite encoding: aStringOrNil! !

!SwazooSeaside class methodsFor: 'service' stamp: 'JM 12/31/2007 20:43'!
start
	"Starts all exisitng sites. Use #startOn: if site on that port not yet exist"
	SwazooServer start! !

!SwazooSeaside class methodsFor: 'service' stamp: 'JM 12/31/2007 20:37'!
startOn: aPort
	| site | 
	site := SeasideSite ensureRegistered: aPort.
	site start! !

!SwazooSeaside class methodsFor: 'service' stamp: 'JM 12/31/2007 20:38'!
stop
	"Stops all sites"
	SwazooServer stop! !

HTTPSetCookieField subclass: #WorkingHTTPSetCookieField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Swazoo'!

!WorkingHTTPSetCookieField commentStamp: 'pmm 7/30/2008 18:06' prior: 0!
A WorkingHTTPSetCookieField is a working implementation of HTTPSetCookieField. The problem with HTTPSetCookieField is that Swazoo developers think it is more important to be compliant with philosophical principles than Firefox and Internet Explorer. The HTTP spec says that any multi valued header field can legally be combined with a ','. But Firefox and Internet Explorer don't support this because the value of set cookie header can legally contain a ','. So this implementaion does not combine set cookie headers. This allows you to set multiple cookies with one response.!

WorkingHTTPSetCookieField subclass: #WorkingHTTPSetCookie2Field
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Swazoo'!

!WorkingHTTPSetCookie2Field commentStamp: 'pmm 7/30/2008 17:44' prior: 0!
A WorkingHTTPSetCookie2Field is a Set-Cookie2 header field, currently only suported by Opera.
!

!WorkingHTTPSetCookie2Field class methodsFor: 'accessing' stamp: 'pmm 7/30/2008 17:38'!
fieldName
	^'Set-Cookie2'! !

!WorkingHTTPSetCookieField methodsFor: 'printing' stamp: 'pmm 7/30/2008 17:58'!
crlfOn: aStream
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!WorkingHTTPSetCookieField methodsFor: 'printing' stamp: 'pmm 7/30/2008 17:58'!
printOn: aStream
	self cookies
		do: [ :each | self printValue: each on: aStream ]
		separatedBy: [ self crlfOn: aStream ]
	! !

!WorkingHTTPSetCookieField methodsFor: 'printing' stamp: 'pmm 7/30/2008 17:50'!
printValue: aString on: aStream
	aStream
		nextPutAll: self name;
		nextPutAll: ': '.
	aStream nextPutAll: aString! !

Site subclass: #SeasideSite
	instanceVariableNames: 'dispatcher codec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Swazoo'!

!SeasideSite commentStamp: 'pmm 11/17/2007 14:48' prior: 0!
I am the implementation of the Seaside server adaptor for Swazoo 2.!

!SeasideSite class methodsFor: 'convenience' stamp: 'pmm 9/30/2007 00:48'!
default
	^self named: self defaultName! !

!SeasideSite class methodsFor: 'accessing-defaults' stamp: 'JM 12/31/2007 19:59'!
defaultHost
	^'*'! !

!SeasideSite class methodsFor: 'accessing-defaults' stamp: 'JM 12/31/2007 20:00'!
defaultIp
	^'*'! !

!SeasideSite class methodsFor: 'accessing-defaults' stamp: 'pmm 9/30/2007 00:43'!
defaultName
	^'seasidesite'! !

!SeasideSite class methodsFor: 'accessing-defaults' stamp: 'pmm 2/27/2008 07:01'!
defaultPort
	^8080! !

!SeasideSite class methodsFor: 'convenience' stamp: 'pmm 9/30/2007 01:18'!
doEncoding: aBoolean
	self default doEncoding: aBoolean! !

!SeasideSite class methodsFor: 'convenience' stamp: 'pmm 10/23/2007 21:39'!
encoding: aStringOrNil
	self default encoding: aStringOrNil! !

!SeasideSite class methodsFor: 'convenience' stamp: 'JM 12/31/2007 20:34'!
ensureRegistered
	^self ensureRegistered: self defaultPort! !

!SeasideSite class methodsFor: 'convenience' stamp: 'pmm 1/1/2008 12:43'!
ensureRegistered: aPort
	^aPort = self defaultPort
		ifTrue: [
			self default isNil
				ifTrue: [ self newDefault ]
				ifFalse: [ self default ] ]
		ifFalse: [
			| site |
			site := SwazooServer singleton siteAnyHostAllInterfacesOnPort: aPort.
			site isNil ifTrue: [
				site := self newOnPort: aPort ].
			site ]! !

!SeasideSite class methodsFor: 'instance creation' stamp: 'JM 12/31/2007 20:22'!
nameForPort: aPort
	^'seasideOn', aPort printString! !

!SeasideSite class methodsFor: 'instance creation' stamp: 'JM 12/31/2007 20:32'!
newDefault
	^(self newNamed: self defaultName)
		host: self defaultHost
		ip: self defaultIp
		port: self defaultPort
	! !

!SeasideSite class methodsFor: 'instance creation' stamp: 'JM 12/31/2007 20:26'!
newOnPort: aPort
	^(self newNamed: (self nameForPort: aPort))
		host: self defaultHost
		ip: self defaultIp
		port: aPort
	! !

!SeasideSite methodsFor: 'serving' stamp: 'jf 11/5/2008 09:40'!
answerTo: aSwazooRequest
	(self isImplemented: aSwazooRequest methodName)
		ifFalse: [ ^self notImplementedResponse ].

	^ SpExceptionContext
		for: [
			| response |
			response := [
				| context |
				context := WARequestContext 
					request: (self convertRequest: aSwazooRequest).
				context codec: self codec.
				self dispatcher handleRequest: context ]
				on: WADynamicVariable
				do: [ :notifiation |
					"hack for SPort bug"
					notifiation resume: notifiation defaultAction ].
			^self convertResponse: response ]
		onAnyExceptionDo: [ :exception | 
			self internalErrorResponseFor: exception ]! !

!SeasideSite methodsFor: 'accessing' stamp: 'pmm 9/13/2008 15:39'!
codec
	codec isNil ifTrue: [
		codec := SeasidePlatformSupport codecForEncoding: 'utf-8' ].
	^codec! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/29/2007 15:04'!
collectionClass
	^OrderedCollection! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 11/21/2007 21:02'!
contentsOfResponse: aResponse
	| contents |
	contents := aResponse contents contents.
	^aResponse contentType isBinary
		ifTrue: [ contents ]
		ifFalse: [ self encode: contents ]! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 7/31/2008 09:23'!
convertCookiesFrom: aWAResonse to: aSwazooResponse
	| cookies setCookieField setCookie2Field |
	cookies := aWAResonse cookies.
	cookies isEmpty ifTrue: [ ^self ].
	
	setCookieField := WorkingHTTPSetCookieField new.
	setCookie2Field := WorkingHTTPSetCookie2Field new.

	cookies do: [ :each |
		setCookieField addCookie: each oldNetscapeString.
		setCookie2Field addCookie: each rfc2965String ].
	
	aSwazooResponse headers addField: setCookieField.
	aSwazooResponse headers addField: setCookie2Field! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2007 20:46'!
convertHeader: aSwazooRequest
	| swazooFields fields |
	swazooFields := aSwazooRequest headers fields.
	fields := Dictionary new: swazooFields size.
	swazooFields keysAndValuesDo: [ :key :field |
		| value |
		value := String streamContents: [ :stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value ].
	^fields! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/28/2008 21:01'!
convertRequest: aSwazooRequest
	^WARequest
		method: aSwazooRequest methodName
		url: (self urlFrom: aSwazooRequest)
		headers: (self convertHeader: aSwazooRequest)
		fields: (self parseFields: aSwazooRequest)
		cookies: (self parseCookies: aSwazooRequest)
		remoteAddress: (self remoteAddressOf: aSwazooRequest)
		nativeRequest: aSwazooRequest! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 10/27/2008 19:55'!
convertResponse: aSeasideResponse
	| swazooResponse |
	aSeasideResponse isNil
		ifTrue:  [ ^self requestHandlingAborted ].
	swazooResponse := HTTPResponse new.
			
	aSeasideResponse headers do: [ :each |
		| field fieldClass |
		fieldClass := HeaderField classForFieldName: each key.
		field := fieldClass newForFieldName: each key withValueFrom: each value.
		swazooResponse headers addField: field ].
	self convertCookiesFrom: aSeasideResponse to: swazooResponse.
	swazooResponse code: aSeasideResponse status asNumber.

	swazooResponse entity: (self contentsOfResponse: aSeasideResponse).
	swazooResponse contentType: aSeasideResponse contentType seasideString.

	aSeasideResponse release.
	^swazooResponse! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/30/2007 01:14'!
decode: aString
	^self codec decode: aString! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 11/20/2007 10:18'!
decodeUrl: aString
	^self codec decodeUrl: aString! !

!SeasideSite methodsFor: 'accessing' stamp: 'pmm 8/30/2007 13:10'!
dispatcher
	^WADispatcher default! !

!SeasideSite methodsFor: 'encoding' stamp: 'pmm 10/23/2007 21:35'!
doEncoding: aBoolean
	self encoding: (aBoolean ifTrue: [ 'utf-8' ] ifFalse: [ nil ])! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/30/2007 01:14'!
encode: aString
	^self codec encode: aString! !

!SeasideSite methodsFor: 'encoding' stamp: 'pmm 9/13/2008 15:39'!
encoding: aStringOrNil
	"valid values for aStringOrNil are:
	nil
		switches off all encoding, like WAKom
	utf-8
		uses the utf-8 fast path by Andreas Raab
	any other encoding supported by Squeak
		uses whatever TextConverter Squeak uses for this encoding"
	codec := SeasidePlatformSupport codecForEncoding: aStringOrNil! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2008 14:40'!
getFieldsOf: aSwazooRequest into: aDictionaryMerger
	aSwazooRequest queries do: [ :each |
		aDictionaryMerger at: each key put: (self decodeUrl: each value) ]! !

!SeasideSite methodsFor: 'serving-private' stamp: 'pmm 9/30/2007 01:13'!
helpResolve: aResolution 
	(self canAnswer and: [ aResolution siteMatch: self ]) ifFalse: [^nil].
	^self answerTo: aResolution request! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/14/2008 15:31'!
internalErrorResponseFor: anError
	^HTTPResponse internalServerError
		entity: 'Internal Error';
		yourself! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2007 20:28'!
isImplemented: aMethodName
	^#('GET' 'POST') includes: aMethodName! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2007 20:27'!
notImplementedResponse
	^HTTPResponse notImplemented! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 7/31/2008 09:19'!
parseCookies: aSwazooRequest
	^WARequestCookie fromString: aSwazooRequest cookie! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2008 14:46'!
parseFields: aSwazooRequest
	| merger |
	merger := WADictionaryMerger new.
	self getFieldsOf: aSwazooRequest into: merger.
	aSwazooRequest isPost ifTrue: [
		self postFieldsOf: aSwazooRequest into: merger ].
	^merger dictionary! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 11/5/2008 15:10'!
postFieldsOf: aSwazooRequest into: aDictionaryMerger
	"break encapsulation"
	aSwazooRequest ensureFullRead.
	aSwazooRequest postData keysAndValuesDo: [ :key :value |
		| previousData data |
		data := value filename isEmptyOrNil
			ifTrue: [ self decodeUrl: value value ]
			ifFalse: [
				WAFile new
					fileName: (self decodeUrl: value filename);
					contentType: value contentType seasideMimeType;
					contents: value value;
					yourself ].
		aDictionaryMerger at: key put: data ]! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 5/11/2008 22:21'!
remoteAddressOf: aSwazooRequest
	"converts a Squeak address (ByteArray) to an IPv4 String"
	| remoteAddress |
	remoteAddress := aSwazooRequest peer.
	remoteAddress isString ifTrue: [
		"so apparently someone has a new VM with IPv6 support"
		^remoteAddress ].
	
	^String streamContents: [ :stream |
		remoteAddress
			do: [ :each | stream print: each ]
			separatedBy: [ stream nextPut: $. ] ]! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 10/27/2008 19:55'!
requestHandlingAborted
	^HTTPResponse internalServerError
		entity: 'Request handling aborted; reload to retry';
		contentType: 'text/plain';
		yourself! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/28/2008 21:01'!
urlFrom: aSwazooRequest
	^(WAUrl absolute: aSwazooRequest uri printString) decodedWith: self codec! !

SeasideSite subclass: #SeasideStreamSite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Swazoo'!

!SeasideStreamSite commentStamp: 'pmm 9/18/2007 06:39' prior: 0!
I do not work yet, do not use me.!

!SeasideStreamSite methodsFor: 'accessing' stamp: 'pmm 9/1/2007 12:28'!
convertRequest: aSwazooRequest
	^(super convertRequest: aSwazooRequest)
		responseStream: aSwazooRequest parent stream;
		yourself! !
